import { NextResponse } from "next/server";
import type { NextRequest } from "next/server";
import { updateUserPfpUrl, getUserPfpUrl } from "~/server/functions";
import { middleware } from "~/middleware";
import fs from "fs/promises";
import path from "path";

const UPLOAD_DIR = path.resolve(process.cwd(), "public/uploads/profile-pictures");

export async function POST(request: NextRequest) {
  console.log("Received POST request to /api/users/updatePfp");
  
  // Apply middleware
  const middlewareResponse = await middleware(request);
  if (middlewareResponse) {
    console.log("Middleware rejected the request");
    return middlewareResponse;
  }

  try {
    console.log("Parsing form data...");
    const formData = await request.formData();
    console.log("Form data parsed successfully");

    const file = formData.get('file') as File | null;
    const appleId = formData.get('appleId') as string | null;

    console.log("Received file:", file ? "Yes" : "No");
    console.log("Received appleId:", appleId);

    if (!file || !appleId) {
      console.log("File or appleId missing");
      return NextResponse.json({ message: "File or appleId missing." }, { status: 400 });
    }

    // Validate file type
    console.log("File type:", file.type);
    if (!file.type.startsWith('image/')) {
      console.log("Invalid file type");
      return NextResponse.json({ message: "Invalid file type. Only images are allowed." }, { status: 400 });
    }

    // Ensure upload directory exists
    console.log("Creating upload directory if it doesn't exist");
    await fs.mkdir(UPLOAD_DIR, { recursive: true });

    // Generate a unique filename
    const fileExtension = path.extname(file.name);
    const fileName = `${appleId}_${Date.now()}${fileExtension}`;
    const filePath = path.join(UPLOAD_DIR, fileName);
    console.log("Generated file path:", filePath);

    // Write file to disk
    console.log("Writing file to disk...");
    const buffer = Buffer.from(await file.arrayBuffer());
    await fs.writeFile(filePath, buffer);
    console.log("File written successfully");

    // Get the relative URL for the uploaded file
    const pfpURL = `/uploads/profile-pictures/${fileName}`;
    console.log("New pfpURL:", pfpURL);

    // Get old pfpURL and remove the old file if it exists
    console.log("Fetching old pfpURL...");
    const oldPfpURL = await getUserPfpUrl(appleId);
    if (oldPfpURL) {
      console.log("Old pfpURL found:", oldPfpURL);
      const oldFilePath = path.join(process.cwd(), 'public', oldPfpURL);
      console.log("Attempting to delete old file:", oldFilePath);
      await fs.unlink(oldFilePath).catch((err) => {
        console.log("Error deleting old file:", err.message);
      });
    } else {
      console.log("No old pfpURL found");
    }

    // Update the user's pfpURL in the database
    console.log("Updating user's pfpURL in the database...");
    await updateUserPfpUrl(appleId, pfpURL);
    console.log("Database updated successfully");

    console.log("Sending successful response");
    return NextResponse.json({ message: "Profile picture updated successfully", pfpURL });
  } catch (error) {
    console.error("Error in updatePFP:", error);
    if (error instanceof Error) {
      console.error("Error details:", error.message);
      return NextResponse.json({ message: error.message }, { status: 500 });
    }
    return NextResponse.json({ message: "Error updating profile picture" }, { status: 500 });
  }
}
